
IS_CYGWIN = $(if $(findstring Windows,$(OS)),true)

####### Compiler, tools and options

CXX      = g++
CXXFLAGS = -pipe -Wall -W -O2 -DHAVE_LIBC
INCPATH  = -I../../src/include 
LINK     = $(CXX)
LIBS	 = -L../../src -lmxcpc$(HWACCEL)

ifeq ($(HWACCEL),IPP)
CXXFLAGS += -DUSE_IPP
LIBS	+= -L$(IPPROOT)/sharedlibs -L$(IPPROOT)/sharedlibs/linux32 -lguide -lippi -lippj
INCPATH	+= -I$(IPPROOT)/include
endif

GNU_TAR  = tar
GZIP     = gzip -9f
COPY     = cp -f
COPY_FILE= $(COPY)
INSTALL_FILE= $(COPY_FILE)
DEL_FILE = rm -f
DEL_DIR  = rmdir
MKDIR	 = mkdir

SEP	 = /
OBJEXT	 = .o


####### Output directory

OBJECTS_DIR = obj/

####### Default rule

first: all

####### Files

MAKEFILE = Makefile.linux-g++
MAKE_FILES = Makefile.linux-g++ Makefile.win32-msvc 

INSTALLDIR = ../../bin/linux/i386-g++

ifeq "$(IS_CYGWIN)" "true"
INSTALLDIR = ../../bin/win32/cygwin-g++
endif

HEADERS = 
SOURCES = $(APP).cpp
OBJECTS = obj/$(APP)$(HWACCEL).o
TARGET  = $(APP)$(HWACCEL)

DISTFILES = $(HEADERS) $(SOURCES) $(MAKE_FILES) obj
DISTFILES += $(addsuffix .expected,$(TESTFILES))


####### Build rules

all: $(TARGET)

$(TARGET): lib obj $(OBJECTS)
	$(LINK) $(LFLAGS) -o $(TARGET) $(OBJECTS) $(LIBS)

obj:
	$(MKDIR) obj

lib:
	$(MAKE) -C ../../src -f $(MAKEFILE)

####### Cleaning

clean:
	-$(DEL_FILE) $(OBJECTS)
	-$(DEL_FILE) *~ core *.core *result
	-$(DEL_DIR) obj

distclean: clean
ifeq "$(IS_CYGWIN)" "true"
	-$(DEL_FILE) $(TARGET).exe
else
	-$(DEL_FILE) $(TARGET)
endif
	$(MAKE) -C ../../src -f $(MAKEFILE) distclean


####### Packaging the sources

dist: ../../../mxcpc_app_$(APP).tar

../../../mxcpc_app_$(APP).tar: $(DISTFILES)
	$(GNU_TAR) -cf $@ -C ../../../ --no-recursion $(addprefix mxcpc/apps/$(APP)/,$^)

####### Dependencies

obj/$(APP)$(HWACCEL).o: $(APP).cpp \
		../../src/include/mxcpcFramewiseMxPEGDecoder.h \
		../../src/include/mxcpcFramewiseMxPEGScanToJPEGConverterSoftwareOnly.h \
		../../src/include/mxcpcMJPEGServerpushWriter.h \
		../../src/include/mxcpcDefaultMxPEGDecoderBackEnd.h \
		../../src/include/mxcpcStreamFile.h \
		../../src/include/mxcpcStreamStandardIn.h \
		../../src/include/mxcpcMxPEGDecoder.h \
		../../src/include/mxcpcStreamDecoder.h \
		../../src/include/mxcpcFramewiseMxPEGScanDecoder.h \
		../../src/include/mxcpc_mxpeg_namespace.h \
		../../src/include/mxcpc_namespace.h \
		../../src/include/mxcpcHuffmanTree.h \
		../../src/include/mxcpcMxPEGScanToJPEGConverterBackEnd.h \
		../../src/include/mxcpcStreamSource.h \
		../../src/include/mxcpcMxPEGDecoderBackEnd.h 
	$(CXX) -c $(CXXFLAGS) $(INCPATH) -o $@ $<



ifdef HWACCEL
obj/$(APP)$(HWACCEL).o: 	../../src/include/mxcpcFramewiseMxPEGScanToJPEGConverter$(HWACCEL).h
endif

####### Install

install: $(TARGET)
	$(INSTALL_FILE) $^ $(INSTALLDIR)

uninstall:  

####### Test

test: $(addsuffix .result,$(TESTFILES))

%.result: ../../test/% $(APP)
	cat $< | ./$(APP) > /dev/null 2> $@
	diff -qs --strip-trailing-cr $(notdir $<).expected $@

