#############################################################################
# MOBOTIX Cross Platform Client (mxcpc)
# Makefile for building mjpeg_pipe on Win32 using the Microsoft toolchain
# Usage example: nmake /f Makefile.win32-msvc
#############################################################################

####### Compiler, tools and options

CXX		=	cl
CXXFLAGS	=	-nologo -Zm200 -EHsc -W3 -w34100 -w34189 -MD -O1 -GX -GX -GR  -DUNICODE -DWIN32
INCPATH		=	-I"..\..\src\include" -I".." 
LINK		=	link
LFLAGS		=	/NOLOGO \
			/NODEFAULTLIB:"libc" /SUBSYSTEM:CONSOLE \
			/LIBPATH:"..\..\src"
LIBS		=	mxcpc$(HWACCEL).lib

!if ("$(HWACCEL)"=="IPP")
CXXFLAGS	=	$(CXXFLAGS) -DUSE_$(HWACCEL)
LIBS		=	$(LIBS) ippi20.lib ippj20.lib
LFLAGS		=	$(LFLAGS) /LIBPATH:"$(HWACCELROOT)\stublib"
INCPATH		=	$(INCPATH) -I"$(IPPROOT)\include"
!endif

COPY_FILE	=       copy
INSTALL_FILE	= 	$(COPY_FILE)
DEL_FILE	=       del /q
DEL_DIR  	= 	rmdir
MKDIR	 	= 	mkdir

####### Files

MAKEFILE	=	Makefile.win32-msvc

INSTALLDIR	=	..\..\bin\win32\msvc

HEADERS 	=	
SOURCES 	=	$(APP).cpp
OBJECTS 	=	obj\$(APP)$(HWACCEL).obj
TARGET		=	$(APP)$(HWACCEL).exe

####### Build rules

all: $(TARGET)

$(TARGET): lib obj $(OBJECTS)
	$(LINK) $(LFLAGS) /OUT:$(TARGET) @<< 
	  $(OBJECTS) $(LIBS)
<<

obj:
	$(MKDIR) obj

lib:
	cd ..\..\src
!ifdef HWACCEL
	SET HWACCEL=$(HWACCEL)
!endif
	$(MAKE) /f $(MAKEFILE) 
	cd $(MAKEDIR)

####### Cleaning

clean: 
	-$(DEL_FILE) obj\*
	-$(DEL_DIR) obj

distclean: clean
	-$(DEL_FILE) $(TARGET)
	cd ..\..\src
	$(MAKE) /f $(MAKEFILE) distclean
	cd $(MAKEDIR)

####### Compile

obj\$(APP)$(HWACCEL).obj: $(APP).cpp \
		..\..\src\include\mxcpcFramewiseMxPEGDecoder.h \
		..\..\src\include\mxcpcFramewiseMxPEGScanToJPEGConverterIPP.h \
		..\..\src\include\mxcpcFramewiseMxPEGScanToJPEGConverterSoftwareOnly.h \
		..\..\src\include\mxcpcMJPEGServerpushWriter.h \
		..\..\src\include\mxcpcDefaultMxPEGDecoderBackEnd.h \
		..\..\src\include\mxcpcStreamFile.h \
		..\..\src\include\mxcpcStreamStandardIn.h \
		..\..\src\include\mxcpcMxPEGDecoder.h \
		..\..\src\include\mxcpcStreamDecoder.h \
		..\..\src\include\mxcpcFramewiseMxPEGScanDecoder.h \
		..\..\src\include\mxcpc_mxpeg_namespace.h \
		..\..\src\include\mxcpc_namespace.h \
		..\..\src\include\mxcpcHuffmanTree.h \
		..\..\src\include\mxcpcMxPEGScanToJPEGConverterBackEnd.h \
		..\..\src\include\mxcpcStreamSource.h \
		..\..\src\include\mxcpcMxPEGDecoderBackEnd.h 
#		..\..\src\include\mxcpcCameraLiveMxPEGFastStreamPOSIX.h \
#		..\..\src\include\mxcpcCameraLiveMxPEGFastStream.h \
	$(CXX) -c $(CXXFLAGS) $(INCPATH) -Fo$@ $(APP).cpp

!ifdef HWACCEL
obj\$(APP)$(HWACCEL).obj: 	..\..\src\include\mxcpcFramewiseMxPEGScanToJPEGConverter$(HWACCEL).h
!endif


####### Install

install: $(TARGET)
	$(INSTALL_FILE) $** $(INSTALLDIR)

uninstall:  

