//           ///          //
//          /////        ////
//         /// XXX     XXX ///
//        ///    XXX XXX    ///     $RCSfile: mjpeg_pipe.cpp,v $  
//       ///       XXX       ///     $Revision: 1.1 $
//      ///      XXX XXX      ///     $Date: 2005/08/09 13:57:26 $
//     ////    XXX     XXX    ////     $Author: cvs-steve $
//    ////                     ////
//   ////  M  O  B  O  T  I  X  ////////////////////////////////////////////////
//  //// Security Vision Systems ///////////////////////////////////////////////

// Copyright (c) 2005, MOBOTIX AG.
// This software is made available under the BSD licence. Please refer 
// to the file LICENCE.TXT contained in this distribution for details.


#include <mxcpcFramewiseMxPEGDecoder.h>

#ifdef USE_IPP
#include <mxcpcFramewiseMxPEGScanToJPEGConverterIPP.h>
#endif
#include <mxcpcFramewiseMxPEGScanToJPEGConverterSoftwareOnly.h>

#include <mxcpcMJPEGServerpushWriter.h>
#include <mxcpcDefaultMxPEGDecoderBackEnd.h>
#include <mxcpcStreamFile.h>
#include <mxcpcStreamStandardIn.h>

#include <cstdio>
#include <cstdlib>

#ifdef HAVE_LIBC
#include <getopt.h>
#else
#include "simpleOptions.h"
#endif


int usage(const char* progname, FILE* stream) {
    fprintf (stream, "%s: Convert an MxPEG stream to MJPEG\n", progname);
    fprintf (stream, "\n");
    fprintf (stream, "Usage: %s [options]\n", progname);
    fprintf (stream, "\n");
    fprintf (stream, "Options:\n");
    fprintf (stream, "-b : Use built-in routines instead of acceleration (Note: acceleration \n");
    fprintf (stream, "     is only available if compiled in. Currently, Intel's \"Integrated \n");
    fprintf (stream, "     Performance Primitves\" library is supported on Windows and Linux.)\n");
    fprintf (stream, "-h : This message\n");
    fprintf (stream, "-s : Insert a MIME separator between JPEG frames (useful within CGI scripts)\n");
    fprintf (stream, "\n");
    fprintf (stream, "Examples:\n");
    fprintf (stream, "cat file1.mxg file2.mxg | %s > file.mjpg\n", progname);
    fprintf (stream, "curl 'http://<camera_ip>/control/faststream.jpg?stream=MxPEG' | %s -s\n", progname);
    fprintf (stream, "C:\\> type file1.mxg | %s > file.mjpg\n", progname);
    fprintf (stream, "\n", progname);
    return 0;
}

int main(int argc, char* argv[]) {

  mxcpcFramewiseMxPEGDecoder *decoder;
  int decoded;
  
  const char* sep = "";
  const char* allowed_opts = "bhs";
  bool ipp_active = false;
#ifdef USE_IPP
  ipp_active = true;
#endif

#ifdef HAVE_LIBC
  int c;
  while ((c = getopt (argc, argv, allowed_opts)) != EOF) {
    switch (c) {
#else
  CSimpleOptions cl( argc, argv, allowed_opts );
  while (cl.next()) {
    switch (cl.option()) {
#endif
    case 'b': // built-in routines
      ipp_active = false;
    case 'h': // help
      usage ("mjpeg_pipe", stdout);
    case 's': // output separator
      sep = "MOBOTIX_Fast_Serverpush";
    }
  }

  mxcpcMJPEGServerpushWriter* writer = new mxcpcMJPEGServerpushWriter (stdout, sep );

  mxcpcFramewiseMxPEGScanDecoder* scandecoder;

#ifdef USE_IPP
  if (ipp_active) {
    scandecoder = new mxcpcFramewiseMxPEGScanToJPEGConverterIPP (writer);
  } else {
#endif
    scandecoder = new mxcpcFramewiseMxPEGScanToJPEGConverterSoftwareOnly (writer);
#ifdef USE_IPP
  }
#endif

  decoder = new mxcpcFramewiseMxPEGDecoder(
					   new mxcpcStreamStandardIn(),
					   new mxcpcDefaultMxPEGDecoderBackEnd(),
					   scandecoder
					   );
								   
  do {
    decoded = decoder->decode(2048);
  } while(decoded);
	
  delete decoder;
}

