//           ///          //
//          /////        ////
//         /// XXX     XXX ///
//        ///    XXX XXX    ///     $RCSfile: mxcpcFramewiseMxPEGScanToJPEGConverterSoftwareOnly.h,v $  
//       ///       XXX       ///     $Revision: 1.1 $
//      ///      XXX XXX      ///     $Date: 2005/08/09 13:57:26 $
//     ////    XXX     XXX    ////     $Author: cvs-steve $
//    ////                     ////
//   ////  M  O  B  O  T  I  X  ////////////////////////////////////////////////
//  //// Security Vision Systems ///////////////////////////////////////////////

// Copyright (c) 2005, MOBOTIX AG.
// This software is made available under the BSD licence. Please refer 
// to the file LICENCE.TXT contained in this distribution for details.


#ifndef __MXCPC_FRAMEWISEMXPEGSCANTOJPEGCONVERTERSOFTWAREONLY_H__
#define __MXCPC_FRAMEWISEMXPEGSCANTOJPEGCONVERTERSOFTWAREONLY_H__


#define MXCPC_FRAMEWISEMXPEGSCANTOJPEGCONVERTER_BITSTREAM_BUFFSIZE 0x100000


#include <mxcpcFramewiseMxPEGScanDecoder.h>
#include <mxcpcHuffmanTree.h>
#include <mxcpc_mxpeg_namespace.h>

#include <cstdio>
#include <cstring>
#include <cstdlib>

class mxcpcMxPEGScanToJPEGConverterBackEnd;



/*! Framewise scan decoder for implementation converting an 
	  <tt>MxPEG</tt> stream to a sequence of JPEG images using
	  no external hw-optimized acceleration routines. 

    \ingroup mxcpc
 
    Does not use the decoder backend ->therefore default backend!
*/
class mxcpcFramewiseMxPEGScanToJPEGConverterSoftwareOnly : public mxcpcFramewiseMxPEGScanDecoder {

 private:
  static const unsigned char 	ZigZag_StoreMap[64];
	static const int 						ZigZagMap[64];
	
	
	//! One DC huffman tables-rules for one component (Y or UV) 
	typedef struct { 
		
		
		/*! 	bits_dc[x] = how many length-codes are 
					encoded with a symbol of length x  - 0 is missing! */ 
    mxcpc::u8 bits_dc[16];      
		
		/*! 	the actual codes for coefficient-lengths from 0 to 11 bits
					the length is the number of bits that are not zero*/
    mxcpc::u8 val_dc[12];
		
	} HuffCompDC;
	
	//! One AC huffman tables-rules for one component (Y or UV) 
	typedef struct { 
		
		/*! max length of code is 16 bits - in fact, 119 of 162 possible combinations are 16 bits ! 
			 				* Including up to 11 significant data bits, 
              * a single non-zero dc-coefficient could therefore 
              * generate up to 27 bits of huffman data */
		mxcpc::u8 bits_ac[16];
		       
    mxcpc::u8 val_ac[162];
		
	} HuffCompAC;
	
	static const HuffCompDC	HuffTbl_Y_UV_DC[2];	
	
	static const HuffCompAC HuffTbl_Y_UV_AC[2];
	
	
	//!  Single entry in table :
	typedef struct {
	
		//! mask, made of rll-encoded symbol(2..16bits) shifted left by (num. of mantissa-bits) 
		unsigned int code_msk:  26;
		//! always 0
    unsigned int fill_0bit:  1;
		//! 2..26, sum of len of rll-encoded symbol + len of mantissa-bits 
    unsigned int bit_len:    5;
		 
	} HuffCodeTblEntry;
	
	
	//! Used to convert a HuffCodeTblEntry -> mxcpc::u32 ! 
	typedef union {
	
		HuffCodeTblEntry 	table_entry;
		mxcpc::u32				u32_word;
	
	} u32t;
		
	/*! One AC huffman table for either Y or UV */
	typedef struct {
    
		//! We index this with symbols 00, 0F, 10..AF and hope gcc won't notice!
		HuffCodeTblEntry ac_tbl[1];
		//! Add 1 to length to be able to use same ptr as for ac-table
    HuffCodeTblEntry dc_tbl[12];
		
		//! Ununsed entry  
    int                     free_13;
		//! Second unused entry [14] is still free    
    int                     free_14;
		//! The valid entries for the ac-table continue here at [15]    
    HuffCodeTblEntry 				dc_tbl_0FtoAF[161];  
	
	} FoldedHuffTbl_DC_AC;

	//! Array of two tables, size = 2*0xB0 words = 1408 bytes : */
	static FoldedHuffTbl_DC_AC Huff_Y_UV[2];
	
	//! This structure holds the buffer state for huffman encoding ...
	typedef struct {
		
		mxcpc::u8* buf_start;       		//!< Addr of buf in memory, word-aligned 
    mxcpc::u8* buf_end1;        		//!< End of buffer + 1, word-aligned */
    mxcpc::u8* buf_current_w_pos;  /*!<  Current buffer pointer, word-aligned if huf_accu is valid,
			     												 			 may be odd after jd_close_huf_buf_stateb was called */	  
                          
    mxcpc::u32 huf_accu;      		//!< Left justified bits not yet emitted
    unsigned int huf_bits_used;   //!< Number of valid bits in huf_accu not yet emitted
    	
	} HuffmanBufferStateStructure;
	
	//! Instance of HuffmanBufferStateStructure hold once!
	static HuffmanBufferStateStructure HuffmanBufferState;
			
	//! Pointer to Huffman encode buffer
  mxcpc::u8* HuffmanEncoderBuffer;
			
 private:
 	
 	mxcpcHuffmanTree *HuffmanTree_Y_DC, 
                   *HuffmanTree_Y_AC; 
									         
  mxcpcHuffmanTree *HuffmanTree_UV_DC, 
                   *HuffmanTree_UV_AC;
			 
  mxcpc::s16 QuantizationTable_Y[64],
             QuantizationTable_UV[64];
						 
	//! Ptr. to decoded frame!
	mxcpc::s16* CoeffBuffer;
 	
	//! Nb. of tiles to handle per frame!
	int TileNumX, TileNumY;
	
	//! The following fields' state is defined to only be valid during scan sweeps
  //! - so they won't show up in the constructor...
  unsigned char *CurrentScanByte,
  	             ScanMask;
  
	int ScanBytesLeft;
  int CoeffInAdditionalBits;
	
  std::FILE *OutStream;
	
	mxcpc_mxpeg::APP0 APP0_Block;
  mxcpc_mxpeg::SOF0 SOF0_Block;
  mxcpc_mxpeg::SOS  SOS_Block;         
  int FramesProcessed;
  
	//! Backend-> Defines how to handle the processed data! 
	mxcpcMxPEGScanToJPEGConverterBackEnd *ConverterBackEnd;

	
 public:
  //! The converter assumes ownership over the converter backend.
  mxcpcFramewiseMxPEGScanToJPEGConverterSoftwareOnly(mxcpcMxPEGScanToJPEGConverterBackEnd *backend);
  ~mxcpcFramewiseMxPEGScanToJPEGConverterSoftwareOnly();
  
 public:
  //! (Re)implemented.
  void setYQuantizationTable(unsigned char *data_bytes);
  //! (Re)implemented.
  void setUVQuantizationTable(unsigned char *data_bytes);
  //! (Re)implemented.
  void performScanSweep(mxcpcFramewiseMxPEGDecoder::UndecodedFrameDescriptor
                         *frame_descriptor);
	
	private:
	
		//! Huffman encoding stuff..
		//! Calculate the huffman tables ..
		void calcFoldedHuffTables(void);
		
		//! Create a huffman lookup table.
		/*!
				\param bits_tbl_off1 Entries 0..15 count no. of symbols 
				                     with 1..16 bits, respectively.
														 
				\param val_tbl Table of symbols that need to be encoded, 
		                   size = sum of entries in bits_tbl_off1.
											 
				\param swap_symbol Set if the swapped symbol is used to access the 
			 										 generated folded table (for ac-coefs).
													 
				\param folded_tbl_p Holds the generated huffman tables!
		*/
		void generateHuffTables1(
    			const mxcpc::u8 * bits_tbl_off1,   
    			const mxcpc::u8 * val_tbl,        
    			int swap_symbol,    
    			HuffCodeTblEntry * folded_tbl_p);
					
		//! Init the Buffer.
		void initHuffmanEncoderBufferState();
		
		void resetHuffmanEncoderBufferState(); 
																								 

	//! We need this often in inner loops of performScanSweep(), so we 
  //! request inline substitution and hope the compiler sufficiently rocks.
  /*!
   *  Only to be called if <tt>ScanBytesLeft > 0</tt>.
   *
   *  Returns whether or not the bit just consumed was set.
   */
  inline bool consumeScanBit(void) {
    
    bool bit_was_set;
    unsigned char last_scan_byte;
    
    bit_was_set = *CurrentScanByte & ScanMask;
    
    ScanMask /= 2;
    if(!ScanMask) {
     
      last_scan_byte = *CurrentScanByte;
    
      CurrentScanByte++;
      ScanBytesLeft--;
      if(last_scan_byte == 0xff) {
        if(*CurrentScanByte == 0x00) {
          if(!ScanBytesLeft) {
            mxcpc::sendStatusMsg("not more scan bytes!");
            std::exit(666);
          }
          else {
            CurrentScanByte++;
            ScanBytesLeft--;
          }
        }
        else {
          mxcpc::sendStatusMsg("unexpected marker encountered!");
          std::exit(666);
        }
      
        //mxcpc::sendStatusMsg("warning: 0xff in datastream, feeding now, checking later!");
      }
      
      ScanMask = 128;
    }
    
    return(bit_was_set);
  }
	
	//! We need this often in inner loops of performScanSweep(), so we 
  //! request inline substitution and hope the compiler sufficiently rocks.
  /*!
   *  Specify as argument the root of the Huffman tree to be used for codeword
   *  recognition.
   * 
   *  Internally checks for <tt>ScanBytesLeft > 0</tt>.
   *
   *  Returns the Huffman leaf node corresponding to the codeword just
   *  consumed or <tt>0</tt> in case of failure. Failure conditions can be
   *  - an invalid bitstring was encountered
   *  - all scan bytes were cosumed before a valid codeword was detected 
   */
  inline const mxcpcHuffmanTree
                 ::Node *consumeHuffmanCodeword(
                          const mxcpcHuffmanTree::Node *current_huffman_node) {
  
    const mxcpcHuffmanTree::Node *last_huffman_node;
    
    do {
         
      if(!ScanBytesLeft) return(0);
     
      last_huffman_node = current_huffman_node;
          
      if(consumeScanBit())
        current_huffman_node = current_huffman_node->Child1;
      else 
        current_huffman_node = current_huffman_node->Child0; 
        
      if(current_huffman_node == last_huffman_node) {
        std::printf(" !!! invalid codeword !!!\n");
        std::exit(666);
        return(0);
      }
              
    } while(!current_huffman_node->IsLeaf);
    
    return(current_huffman_node);
  }

	
  //! We need this often in inner loops of performScanSweep(), so we 
  //! request inline substitution and hope the compiler sufficiently rocks.
  /*!
   *  Specify as argument the number <tt>n >= 1</tt> of additional bits to 
   *  read. 
   *  
   *  The coefficient represented by the read bits will be stored to the
   *  class member variable <tt>CoeffInAdditionalBits</tt>.
   *
   *  The method internally checks for <tt>ScanBytesLeft > 0</tt>.
   *
   *  It returns <tt>false</tt> on success, and <tt>true</tt> if all scan
   *  bytes were cosumed before a valid codeword was detected. 
   */
  inline bool consumeAdditionalBits(int n) {
  
    bool positive;
    int base;
    static int bases[] = { -666, -666, 2, 4, 8, 16, 32, 64, 128, 256, 512 };    
    
    // read first bit...
    if(!ScanBytesLeft) return(true);
    positive = consumeScanBit();
    if(n == 1) {
      if(positive) CoeffInAdditionalBits =  1;
      else         CoeffInAdditionalBits = -1;
      return(false);
    }
    
    // read remaining bits...
    base = bases[n];
    if(!positive) {
      base *= 2;
      base = -base;
      base++;
    }
    n--;
    CoeffInAdditionalBits = 0;
    while(n) {
    
      if(!ScanBytesLeft) return(true);
      
      CoeffInAdditionalBits *= 2;
      
      if(consumeScanBit()) 
        CoeffInAdditionalBits += 1;
       
      n--;
    }
    CoeffInAdditionalBits += base;  
    
    return(false);
  }

	
	/*!
			This Method takes a 8x8 sub-MCU and preforms a swift Huffman Encoding.
			
			\param coef_matrix						The 8x8 input coefficient matrix!
			\param last_dc_coeff					Ptr to a list of the last DC coefficients.
			\param huff_code_tbl_entry    A huffman table.
			\param hbuf_state_ptr				 	Ptr to global buffer state.
	*/
	inline void encodeSubMCUTiles(mxcpc::s16* coef_matrix,
																mxcpc::s16* last_dc_coeff,
																FoldedHuffTbl_DC_AC* huff_code_tbl_entry,
																HuffmanBufferStateStructure* hbuf_state_ptr)				{
																
		
		// Fetch settings state from global context!
		unsigned int 	huff_bits_used 	= hbuf_state_ptr->huf_bits_used;
		mxcpc::u32		huff_accu_32		= hbuf_state_ptr->huf_accu;
		
		u32t	union_accu_tmp;
		int		total_bits=0;
		int 	bits=0; 
		/*


			Convert coefficients in Huffman codewords ...

			___ DC coefficient
			| 
			v
			34 12 3 0 0 5 0 4 0 1 ....
 				 ^        ^
				 |        |_______ "3" AC coefficients != 0,  two(R) 0-coefficients before that -> RB = 23
				 |                                          	Bitlength(B) from 5=101_b = 3
				 |
				 |
				 |___ "1" AC coefficient
				
		*/
		
		//-------------------------------------------------------------------------
		// First start encoding DC component ... 									
		//-------------------------------------------------------------------------
		
		// Container for the current huffman codeword indexed by the value of delta.
		mxcpc::u32 dc_encoded_word;
		// First element in list is always the DC coefficient!
		mxcpc::s16 delta =  coef_matrix[0] - (*last_dc_coeff);
		*last_dc_coeff = coef_matrix[0];
			
		// Delta should be in boundaries ...
		if ((delta < -2046) || (delta > 2046)) {
				mxcpc::sendStatusMsg("Out of Range");
				std::exit(666);
		}
		
		/* 0000 bbbb , 1 <= b <= 10 */
		if( delta ==0 ) bits=0;
		if(delta  > 0 )bits = returnBitsInCoef(delta );
		if(delta  < 0 )bits = returnBitsInCoef(-delta ); 
		
		union_accu_tmp.table_entry = (*huff_code_tbl_entry).dc_tbl[bits];
		dc_encoded_word							 = union_accu_tmp.u32_word;
		
		/* build one's complement */
		if (delta  < 0) delta  = (delta  -1 ) & (0x7fffffff >> (31 - bits)); /* 1111 1011 -> ____ _010 */
		
		/*  tttt.t0hh_hhhh.hhhh_hhhh.hhoo.oooo.oooo 
				^      ^                   ^
				|      |                   |___ooo = <bits> zeros, will be overwritten with mantissa later 
							hhhhh=huffman-code,
			tttt.t _____ number of hhh + ooo bits, 2 <= h <= 16, 0 <= o <= 10, 2 <= ttttt <= 26 */
		dc_encoded_word = dc_encoded_word | delta ;    /* ...000hhhhhccc */
		total_bits   = dc_encoded_word >> 27;
		
		// kill tttt.t(32-total_bits always > 5) and left align the new bit stream ...
		dc_encoded_word = dc_encoded_word << (32 - total_bits);           /*   hhhh.hccc_0000.0000_0000.0000_0000.000 */
		
		
																																		 	/*   ####.## -> huff_bits_used = 6           */       
		huff_accu_32 = huff_accu_32 | (dc_encoded_word >> huff_bits_used);/*   aaaa.aaoo_oooo.oooo_oooo.oooo_oooo.oooo *
																																			 * | 0000.00hh_hhhc.cc00_0000.0000_0000.0000 
																																			 * -------------------------------------------
														                                     			 * = aaaa.aahh_hhhc.cc00_0000.0000_0000.0000
																																			 *   ####.####_####.## -> new huff_bits_usded = 6+8 = 14 */
		huff_bits_used += total_bits;
		
		
		// Special case accu full ...
		if( huff_bits_used >= 32 ){
			
			// Write accu (32 bit) to output_bitstream_buff ...
			// this may generate more than 4 bytes becuase of zero-stuffing
			
			copyAccuToGlobalHuffBuffer(	huff_accu_32,	
															  	hbuf_state_ptr);
			huff_bits_used -= 32;
			huff_accu_32 = dc_encoded_word << (total_bits - huff_bits_used); /* (hhhh) <-  hccc.oooo_oooo.oooo_oooo.oooo_oooo.oooo */
			
		}
									
		//-------------------------------------------------------------------------																		
		// Now start encoding AC coeff ...
		//-------------------------------------------------------------------------
		
		int		run_len=0;						/* count zeros */
		
		
		for(int ac_coef=1; ac_coef<=63; ac_coef++){
	
		
			mxcpc::u32 new_bit_accu; 	/* used to "or" symbol and mantissa, must be u32 rather than int
                       					   so a (new_bit_accu >> n) never shifts in 1 from bit 31 */
    	bits=0;    						    /* number of bits in coef,  0000 bbbb , 1 <= b <= 10 */
    	int symbol=0; 						/* symbol, (run_len << 4) | bits = ..00 rrrr bbbb , 0 <= rr <= 15   */
						
    	int c = coef_matrix[ZigZagMap[ac_coef]];
   
			if(c == 0 ){
			
				run_len +=1;
						
			}
			else{

				// max number of zeros to encoding table is 15
				while (run_len > 15) {
				        	
					mxcpc::u32 u32_zero_lenght;
										
					union_accu_tmp.table_entry 	= (*huff_code_tbl_entry).ac_tbl[0x0f];
					u32_zero_lenght   					= union_accu_tmp.u32_word;
													
					total_bits   = u32_zero_lenght >> 27;
							
					u32_zero_lenght = u32_zero_lenght << (32 - total_bits);
					
					huff_accu_32 = huff_accu_32 | ( u32_zero_lenght >> huff_bits_used); 
			
					huff_bits_used += total_bits;
											
					if( huff_bits_used >= 32 ){

				  	copyAccuToGlobalHuffBuffer(	huff_accu_32,	
																		  	hbuf_state_ptr);
						
						/* ex: huff_bits_used = 4, total_bits=8 */
						huff_bits_used -= 32;
						/* (hhhh) <-  hccc.oooo_oooo.oooo_oooo.oooo_oooo.oooo */
						huff_accu_32 = u32_zero_lenght << (total_bits - huff_bits_used); 
						
						
	  			}
					run_len -= 16;
        }
				
				/* 0000 bbbb , 1 <= b <= 10 */
				if(c >=0 )bits = returnBitsInCoef(c);
				else { bits = returnBitsInCoef(-c); }
				
								    
				/* symbol, (run_len << 4) | bits = ..00 rrrr bbbb , 0 <= rr <= 15   */  
				symbol = (bits << 4) + run_len;
			
								   
				/*   tttt.t0hh_hhhh.hhhh_hhhh.hhoo.oooo.oooo 
						 ^      ^                   ^
						 |      |                   |___ooo = <bits> zeros, will be overwritten with mantissa later 
										hhhhh=huffman-code,
						tttt.t _____ number of hhh + ooo bits, 2 <= h <= 16, 0 <= o <= 10, 2 <= ttttt <= 26 */
				union_accu_tmp.table_entry = (*huff_code_tbl_entry).ac_tbl[symbol]; 
				new_bit_accu 							 = union_accu_tmp.u32_word;
				
				
				/* build one's complement */
				if (c < 0) c = (c -1 ) & (0x7fffffff >> (31 - bits)); /* 1111 1011 -> ____ _010 */
					
				new_bit_accu = new_bit_accu | c;    /* ...000hhhhhccc */
				total_bits   = new_bit_accu >> 27;  /* total_bits = ..00t.tttt (2..26) */
				
				
				// kill tttt.t(32-total_bits always > 5) and left align the new bit stream ...
				new_bit_accu = new_bit_accu << (32 - total_bits);           /*   hhhh.hccc_0000.0000_0000.0000_0000.000 */

					
				/* now we must append the new bit-stream of <total_bits> length in new_bit_accu to the huffman bitstream */

																																				 /*   ####.## -> huff_bits_used = 6           */       
				huff_accu_32 = huff_accu_32 | (new_bit_accu >> huff_bits_used);  /*   aaaa.aaoo_oooo.oooo_oooo.oooo_oooo.oooo *
																																					* | 0000.00hh_hhhc.cc00_0000.0000_0000.0000 
																																					* -------------------------------------------
																                                     			* = aaaa.aahh_hhhc.cc00_0000.0000_0000.0000
																																					*   ####.####_####.## -> new huff_bits_usded = 6+8 = 14
																																					*/
				huff_bits_used += total_bits;
					
				if (huff_bits_used >= 32) {
					copyAccuToGlobalHuffBuffer(	huff_accu_32,	
																	  	hbuf_state_ptr);
					huff_bits_used -= 32;                                         /* ex: huff_bits_used = 4, total_bits=8 */
					huff_accu_32 = new_bit_accu << (total_bits - huff_bits_used); /* (hhhh) <-  hccc.oooo_oooo.oooo_oooo.oooo_oooo.oooo */
				
				}
       			
				run_len =0;			
			}
			

		}
		
		// write EOB ...
		if(run_len != 0){
			
			mxcpc::u32 eob_tag;
			
			union_accu_tmp.table_entry 	= (*huff_code_tbl_entry).ac_tbl[0x00];
			eob_tag 										= union_accu_tmp.u32_word;
					
			total_bits   = eob_tag >> 27;
			
			eob_tag = eob_tag << (32 - total_bits);
			 
			huff_accu_32 = huff_accu_32 | (eob_tag >> huff_bits_used); 
			
			huff_bits_used += total_bits;
			
			if (huff_bits_used >= 32) {
					copyAccuToGlobalHuffBuffer(	huff_accu_32,	
																	  	hbuf_state_ptr);
					huff_bits_used -= 32;                                         /* ex: huff_bits_used = 4, total_bits=8 */
					huff_accu_32 = eob_tag << (total_bits - huff_bits_used); /* (hhhh) <-  hccc.oooo_oooo.oooo_oooo.oooo_oooo.oooo */
			}		
									
		}
		
		// update global buffer ..
		hbuf_state_ptr->huf_bits_used 	= huff_bits_used;
		hbuf_state_ptr->huf_accu			 	= huff_accu_32;
				
	}
	
	/*!
				Copies a word into the global huffman encoding buffer!
	
				\param huff_accu_32 Input word.
				\param huff_buf_state_ptr Ptr. to global buffer.
	*/
	static inline void copyAccuToGlobalHuffBuffer( mxcpc::u32										huff_accu_32,
																								HuffmanBufferStateStructure*  huff_buf_state_ptr){
																					
		mxcpc::u8  b;
    mxcpc::u8* p = huff_buf_state_ptr->buf_current_w_pos;

    if ((p+8) >= huff_buf_state_ptr->buf_end1) {
       mxcpc::sendStatusMsg("Huffman Buffer Overflow!");
			 std::exit(666);
    }
		
		
    /* Output four bytes from huff_accu_32 with MSByte first, pad each 0xff with 0x00 (zero-stuffing)
     * so that control-codes beginning with 0xff can be distinguished from bitstream 
     */
    /*                                                              huffaccu = 76543210; 					*/
	
    b = (huff_accu_32 >> 24); 											*p++ = b;		/* b = 0x76 */
                                     if (b == 0xff) *p++ = 0x00;
																		 
																			 																
    b = (huff_accu_32 >> 16) & 0xff;                *p++ = b;    /* b = 0x54 */
                                     if (b == 0xff) *p++ = 0x00;
																		 
																			 																	
    b = (huff_accu_32 >>  8) & 0xff;                *p++ = b;    /* b = 0x32 */
                                     if (b == 0xff) *p++ = 0x00;
		
		 
    b = (huff_accu_32      ) & 0xff;                *p++ = b;    /* b = 0x10 */
                                     if (b == 0xff) *p++ = 0x00;
																		 
    huff_buf_state_ptr->buf_current_w_pos = p;
		
	}

	/*!
				Closing down huffman encoding stream.
	
	*/
	static int shutDownHuffAccu( ){
		
		mxcpc::u8  b;
    mxcpc::u8* p = HuffmanBufferState.buf_current_w_pos;

    if ((p+8) >= HuffmanBufferState.buf_end1) {
       mxcpc::sendStatusMsg("Huffman Buffer Overflow!");
			 std::exit(666);
    }
		
		if(HuffmanBufferState.huf_bits_used == 2 && HuffmanBufferState.huf_accu == 0 ){

			b = 0x0 ; 		
			*p++ = b;		
			
			HuffmanBufferState.buf_current_w_pos = p;																								
			
		}												
		else{
			while (HuffmanBufferState.huf_accu) {
					
					b = (HuffmanBufferState.huf_accu >> 24); 		*p++ = b;		
																											if (b == 0xff) *p++ = 0x00;
					HuffmanBufferState.huf_accu <<= 8;
			}
			
			HuffmanBufferState.buf_current_w_pos = p;
			}
		return(0);
	}
		
/*	static int c_to_bits[0,1,2,2,
	                     3, 3, 3, 3,
											 4, 4, 4, 4, 4, 4, 4, 4] ;*/
	
	/*!
			Number of bits from ....
			
			\param c <- this!
	*/
	static inline int returnBitsInCoef(mxcpc::s16 c){
		
//		if (c <16) return (c_to_bits[c]);
		int bits = 1;
		
		while ( c >>= 1 ){ bits +=1;}
		
		return (bits);
	
	}

		
};


#endif   // __MXCPC_FRAMEWISEMXPEGSCANTOJPEGCONVERTERSOFTWAREONLY_H__
