//           ///          //
//          /////        ////
//         /// XXX     XXX ///
//        ///    XXX XXX    ///     $RCSfile: mxcpcHuffmanTree.h,v $  
//       ///       XXX       ///     $Revision: 1.3 $
//      ///      XXX XXX      ///     $Date: 2005/08/09 13:57:26 $
//     ////    XXX     XXX    ////     $Author: cvs-steve $
//    ////                     ////
//   ////  M  O  B  O  T  I  X  ////////////////////////////////////////////////
//  //// Security Vision Systems ///////////////////////////////////////////////

// Copyright (c) 2005, MOBOTIX AG.
// This software is made available under the BSD licence. Please refer 
// to the file LICENCE.TXT contained in this distribution for details.


#ifndef __MXCPC_HUFFMANTREE_H__
#define __MXCPC_HUFFMANTREE_H__




//! Huffman tree with <tt>unsigned char</tt> valued leaves - used for 
//! <tt>MxPEG</tt> decoding.
/*!
 *  \ingroup mxcpc
 */
class mxcpcHuffmanTree {

 public:
  struct Node {
    Node *Child0, *Child1;
    bool IsLeaf;
    unsigned char Value;
  };
  
 private:
  Node *RootNode; 
  Node *GroundNode;

 public:
  mxcpcHuffmanTree();
  ~mxcpcHuffmanTree();
  
 public:
  //! Returns the Huffman tree's root node.
  const Node *getRoot(void);
   
  //! Clears the Huffman tree.
  void clear(void);
  
	//! Reconfigures the Huffman tree as specified in the given table 
  //! (JPEG Huffman table format).
  /*!
   *  Returns <tt>false</tt> on success, and <tt>true</tt> otherwise.
   *  In case of failue, the tree is cleared.
   */
  bool configureFromTable(const unsigned char *table, int max_path_len = 16);
  
	//! Adds a leaf that is reached from the root via a path of length 
  //! <tt>path_len</tt> bits.
  /*! 
   *  Returns <tt>false</tt> on success, and <tt>true</tt> otherwise.
   *  In any case the tree remains in defined state.
   */
  bool addLeaf(int path_len, unsigned char value);
  
	//! Returns the number of leaves in the tree.
  int countLeaves(void);
  
 private:
  //! Wastes the specified subtree.
  void deleteSubTree(Node *subtree_root);
  //! Adds to the specified subtree a leaf that is reached from the subtree 
  //! root via a path of length <tt>path_len</tt> bits.
  /*! 
   *  Returns <tt>false</tt> on success, and <tt>true</tt> otherwise.
   *  In any case the tree remains in defined state.
   */
  bool addLeaf(Node *subtree_root, int path_len, Node *leaf);
  //! Recursively makes unused child pointers of all inner nodes point to the
  //! special <i>ground node</i>, from which tree traversals never recover.
  /*!
   *  So if invalid bitstreams are encountered the tree traversal will forever
   *  be caught in the ground node and the Huffman decoder using the tree  will
   *  simply cease to produce video tiles - without the need of timeconsuming
   *  code to detect such invalid bitstreams.  
   */
  void groundUnusedPaths(Node *subtree);
  //! Private helper method.
  int countLeaves(Node *subtree);
};



#endif   // __MXCPC_HUFFMANTREE_H__
