//           ///          //
//          /////        ////
//         /// XXX     XXX ///
//        ///    XXX XXX    ///     $RCSfile: mxcpcMxPEGDecoderBackEnd.h,v $  
//       ///       XXX       ///     $Revision: 1.6 $
//      ///      XXX XXX      ///     $Date: 2005/08/09 13:57:26 $
//     ////    XXX     XXX    ////     $Author: cvs-steve $
//    ////                     ////
//   ////  M  O  B  O  T  I  X  ////////////////////////////////////////////////
//  //// Security Vision Systems ///////////////////////////////////////////////

// Copyright (c) 2005, MOBOTIX AG.
// This software is made available under the BSD licence. Please refer 
// to the file LICENCE.TXT contained in this distribution for details.


#ifndef __MXCPC_MXPEGDECODERBACKEND_H__
#define __MXCPC_MXPEGDECODERBACKEND_H__



#include <mxcpc_namespace.h>



//! Interface (baseclass) to MxPEG decoder backends.
/*! 
 *  \ingroup mxcpc
 *
 *  Backends decide for what is actually to be done with video tiles and 
 *  audio segments decoded from a MxPEG stream.
 *
 */
class mxcpcMxPEGDecoderBackEnd {

 public:
  virtual ~mxcpcMxPEGDecoderBackEnd();

 public:
  
  //! To inform the backend that the stream switched to another video 
  //! resolution.
  /*!
   *  Guaranteed to be called at least once before the first tiles arrive.
   *
   *  Guaranteed to be called with both dimensions being positive.
   */
  virtual void videoResolutionChanged(int width, int height) = 0;
  
  //! To inform the backend that while decoding the current frame, an error
  //! condition was encountered.
  /*!
   *  Can be called multiple times while decoding of a frame is in progress.
   *  Processing of current frame is to be cosidered as finished only when
   *  the frameComplete() method is invoked.
   */
  virtual void errorEncountered(void) = 0;
  
  //! To inform the backend that decoding of the current frame was finished
  //! and that - as a result - the following video tiles belong to the next
  //! frame.
  /*!
   *  \note A call to this method does not mean that the frame was decoded 
   *  <i>successfully</i>, see errorEncountered().
   */
  virtual void frameComplete(void) = 0;
  
  //! By calling this method the decoder asks the backend where to store the
  //! decoded pixel data for the next video tile.
  /*!
   *  The pixel data will be written out RGBRGBRGB... with one byte per color
   *  channel. Between rows, a certain amount of memory is skipped according
   *  to the specified row stepping which is given in <tt>u32</tt> units.
   *  It is perfectly legal to specify negative values here, in fact this is 
   *  how it will be handled in most cases when <tt>OpenGL</tt> is involved in
   *  the actual rendering, because here, the origin of 2D image data is 
   *  usually located in the lower/left corner.
   */
  virtual void provideVideoTilePixelTarget(int tile_x, int tile_y,
                                            mxcpc::u32 **target_ptr,
                                            int *row_stepping_ptr)   = 0; 
  //! To inform the backend about a received video tile. 
  /*! It has been stored to the pixel target provided by the backend 
   *  through a previous call to provideVideoTilePixelTarget(). 
   */
  virtual void videoTileReceived(void) = 0; 
};



#endif   // __MXCPC_MXPEGDECODERBACKEND_H__
