//           ///          //
//          /////        ////
//         /// XXX     XXX ///
//        ///    XXX XXX    ///     $RCSfile: mxcpcCameraLiveMxPEGFastStreamPOSIX.cpp,v $  
//       ///       XXX       ///     $Revision: 1.6 $
//      ///      XXX XXX      ///     $Date: 2005/08/09 13:57:26 $
//     ////    XXX     XXX    ////     $Author: cvs-steve $
//    ////                     ////
//   ////  M  O  B  O  T  I  X  ////////////////////////////////////////////////
//  //// Security Vision Systems ///////////////////////////////////////////////



#include <mxcpcCameraLiveMxPEGFastStreamPOSIX.h>
#include <mxcpc_exceptions.h>

#include <cstdio>

#include <sys/types.h>

#ifdef WIN32
#define in_addr_t unsigned long
#define socklen_t int
#include <windows.h>
#include <io.h>
#else
#include <sys/socket.h>
#include <netinet/in.h>
#include <unistd.h>
#endif
#include <fcntl.h>



mxcpcCameraLiveMxPEGFastStreamPOSIX::mxcpcCameraLiveMxPEGFastStreamPOSIX(int ip1,
                                                                         int ip2, 
                                                                         int ip3, 
                                                                         int ip4, 
                                                                         int port,
                                                                         int fps)
                                      : mxcpcCameraLiveMxPEGFastStream(ip1,
                                                                       ip2,
                                                                       ip3,
                                                                       ip4,
                                                                       port,
                                                                       fps)        {
                            
  in_addr_t server_ip;
  struct sockaddr_in server_socket_info;
#ifndef WIN32
  int flags;
#endif
  char txt[100], *txt_ptr;
  int len, written;
  
  
  Socket = -1;
  try {
  
    // get a socket
    Socket = socket(PF_INET, SOCK_STREAM, 0);
    if(Socket < 0) 
      throw mxcpcIOFailure();
   
    // connect to server...
    server_socket_info.sin_family = AF_INET;
    server_socket_info.sin_port   = htons(port);
    server_ip = 0;
    server_ip += ip4;   server_ip *= 256;
    server_ip += ip3;   server_ip *= 256;
    server_ip += ip2;   server_ip *= 256;
    server_ip += ip1;
    server_socket_info.sin_addr.s_addr = server_ip;
    if(::connect(Socket, (const struct sockaddr *)&server_socket_info,
                         (socklen_t)sizeof(server_socket_info))) 
      throw mxcpcIOFailure();
      
    // make socket nonblocking...
#ifdef WIN32 // WIN32 oughta be POSIX, eh?
    unsigned long i_dummy = 1;
    ioctlsocket(Socket, FIONBIO, &i_dummy);
#else
    flags = fcntl(Socket, F_GETFL);
    flags |= O_NONBLOCK;
    fcntl(Socket, F_SETFL, flags);  
#endif  
    // do HTTP GET request 
    if(fps < 1) fps = 4;
    std::sprintf(txt, "GET /control/faststream.jpg?noaudio&fps=%d HTTP/1.0\n\n",
                 fps);
    len = std::strlen(txt);
    txt_ptr = txt;
    do {
      written = write(Socket, txt_ptr, len);
      if(written < 0) throw mxcpcIOFailure();
      len -= written;
      txt_ptr += written;
    } while(len);
      
  } catch(std::exception &e) {
    
      MXCPC_RETHROW_UNHANDLED_EXCEPTION(e);

      if(Socket >= 0) {
#ifdef WIN32      
	closesocket(Socket);
#else
	shutdown(Socket, SHUT_RDWR);
#endif
      }
    
      throw;
    }
}


mxcpcCameraLiveMxPEGFastStreamPOSIX::~mxcpcCameraLiveMxPEGFastStreamPOSIX() {

  if(Socket >= 0) {
#ifdef WIN32      
    closesocket(Socket);
#else
    shutdown(Socket, SHUT_RDWR);
#endif
  }
}



int mxcpcCameraLiveMxPEGFastStreamPOSIX::fetchBytes(unsigned char *buffer, 
                                                    int num)               {
                                                    
   int result;
   
   result = read(Socket, buffer, num);
   if(result < 0) result = 0;
   
   return(result);
}
