//           ///          //
//          /////        ////
//         /// XXX     XXX ///
//        ///    XXX XXX    ///     $RCSfile: mxcpcFramewiseMxPEGScanDecoderSoftwareOnly.h,v $  
//       ///       XXX       ///     $Revision: 1.2 $
//      ///      XXX XXX      ///     $Date: 2005/08/09 13:57:26 $
//     ////    XXX     XXX    ////     $Author: cvs-steve $
//    ////                     ////
//   ////  M  O  B  O  T  I  X  ////////////////////////////////////////////////
//  //// Security Vision Systems ///////////////////////////////////////////////

// Copyright (c) 2005, MOBOTIX AG.
// This software is made available under the BSD licence. Please refer 
// to the file LICENCE.TXT contained in this distribution for details.


#ifndef __MXCPC_FRAMEWISEMXPEGSCANDECODERSOFTWAREONLY_H__
#define __MXCPC_FRAMEWISEMXPEGSCANDECODERSOFTWAREONLY_H__



#include <mxcpcFramewiseMxPEGScanDecoder.h>
#include <mxcpcHuffmanTree.h>
#include <mxcpc_namespace.h>

#include <cstring>
#include <cstdlib>
#include <cstdio>



//! Software-only decoding module to be used with the framewise (frame 
//! prefetching) <tt>MxPEG</tt> decoder facility implemented by
//! mxcpcFramewiseMxPEGDecoder.
/*!
 *  \ingroup mxcpc
 *
 *  Through this portable scan decoder implementation we establish baseline
 *  <tt>MxPEG</tt> decoding on all three major target platforms in a fashion
 *  entirely independent of additional support libraries like Intel's IPP.
 *  However, where available the performance benefits of such support libraries
 *  can be exploited by plugging into the mxcpcFramewiseMxPEGDecoder another
 *  scan decoder implementation, for example the 
 *  mxcpcFramewiseMxPEGScanDecoderIPP.
 *
 *  \author Kai Hergenroether
 */
class mxcpcFramewiseMxPEGScanDecoderSoftwareOnly : public mxcpcFramewiseMxPEGScanDecoder {

 private:
  static const int ZigZagMap[64];
  static short iclip[1024];
  static short *iclp;
        
 private:
  mxcpcHuffmanTree *HuffmanTree_Y_DC, 
                   *HuffmanTree_Y_AC;         
  mxcpcHuffmanTree *HuffmanTree_UV_DC, 
                   *HuffmanTree_UV_AC;
									 
  mxcpc::s16 QuantizationTable_Y[64],
             QuantizationTable_UV[64];
                    
  // The following fields' state is defined to only be valid during scan sweeps
  // - so they won't show up in the constructor...
  unsigned char *CurrentScanByte,
                ScanMask;
  int ScanBytesLeft;
  int CoeffInAdditionalBits;      
  
 public:
  mxcpcFramewiseMxPEGScanDecoderSoftwareOnly();
  ~mxcpcFramewiseMxPEGScanDecoderSoftwareOnly();
 
 public:
  //! (Re)implemented.
  void setYQuantizationTable(unsigned char *data_bytes);
  //! (Re)implemented.
  void setUVQuantizationTable(unsigned char *data_bytes);
  //! (Re)implemented.
  void performScanSweep(mxcpcFramewiseMxPEGDecoder::UndecodedFrameDescriptor
                         *frame_descriptor);        
                         
 private:
  //! We need this often in inner loops of performScanSweep(), so we 
  //! request inline substitution and hope the compiler sufficiently rocks.
  /*!
   *  Only to be called if <tt>ScanBytesLeft > 0</tt>.
   *
   *  Returns whether or not the bit just consumed was set.
   */
  inline bool consumeScanBit(void) {
    
    bool bit_was_set;
    unsigned char last_scan_byte;
    
    bit_was_set = *CurrentScanByte & ScanMask;
    
    ScanMask /= 2;
    if(!ScanMask) {
     
      last_scan_byte = *CurrentScanByte;
    
      CurrentScanByte++;
      ScanBytesLeft--;
      if(last_scan_byte == 0xff) {
        if(*CurrentScanByte == 0x00) {
          if(!ScanBytesLeft) {
            mxcpc::sendStatusMsg("not more scan bytes!");
            std::exit(666);
          }
          else {
            CurrentScanByte++;
            ScanBytesLeft--;
          }
        }
        else {
          mxcpc::sendStatusMsg("unexpected marker encountered!");
          std::exit(666);
        }
      
        //mxcpc::sendStatusMsg("warning: 0xff in datastream, feeding now, checking later!");
      }
      
      ScanMask = 128;
    }
    
    return(bit_was_set);
  }
  
  //! We need this often in inner loops of performScanSweep(), so we 
  //! request inline substitution and hope the compiler sufficiently rocks.
  /*!
   *  Specify as argument the root of the Huffman tree to be used for codeword
   *  recognition.
   * 
   *  Internally checks for <tt>ScanBytesLeft > 0</tt>.
   *
   *  Returns the Huffman leaf node corresponding to the codeword just
   *  consumed or <tt>0</tt> in case of failure. Failure conditions can be
   *  - an invalid bitstring was encountered
   *  - all scan bytes were cosumed before a valid codeword was detected 
   */
  inline const mxcpcHuffmanTree
                 ::Node *consumeHuffmanCodeword(
                          const mxcpcHuffmanTree::Node *current_huffman_node) {
  
    const mxcpcHuffmanTree::Node *last_huffman_node;
    
    do {
         
      if(!ScanBytesLeft) return(0);
     
      last_huffman_node = current_huffman_node;
          
      if(consumeScanBit())
        current_huffman_node = current_huffman_node->Child1;
      else 
        current_huffman_node = current_huffman_node->Child0; 
        
      if(current_huffman_node == last_huffman_node) {
        std::printf(" !!! invalid codeword !!!\n");
        std::exit(666);
        return(0);
      }
              
    } while(!current_huffman_node->IsLeaf);
    
    return(current_huffman_node);
  }
  
  //! We need this often in inner loops of performScanSweep(), so we 
  //! request inline substitution and hope the compiler sufficiently rocks.
  /*!
   *  Specify as argument the number <tt>n >= 1</tt> of additional bits to 
   *  read. 
   *  
   *  The coefficient represented by the read bits will be stored to the
   *  class member variable <tt>CoeffInAdditionalBits</tt>.
   *
   *  The method internally checks for <tt>ScanBytesLeft > 0</tt>.
   *
   *  It returns <tt>false</tt> on success, and <tt>true</tt> if all scan
   *  bytes were cosumed before a valid codeword was detected. 
   */
  inline bool consumeAdditionalBits(int n) {
  
    bool positive;
    int base;
    static int bases[] = { -666, -666, 2, 4, 8, 16, 32, 64, 128, 256, 512 };    
    
    // read first bit...
    if(!ScanBytesLeft) return(true);
    positive = consumeScanBit();
    if(n == 1) {
      if(positive) CoeffInAdditionalBits =  1;
      else         CoeffInAdditionalBits = -1;
      return(false);
    }
    
    // read remaining bits...
    base = bases[n];
    if(!positive) {
      base *= 2;
      base = -base;
      base++;
    }
    n--;
    CoeffInAdditionalBits = 0;
    while(n) {
    
      if(!ScanBytesLeft) return(true);
      
      CoeffInAdditionalBits *= 2;
      
      if(consumeScanBit()) 
        CoeffInAdditionalBits += 1;
       
      n--;
    }
    CoeffInAdditionalBits += base;  
    
    return(false);
  }
  
  
  //! Stolen.
  void idctrow(short *blk);
  //! Stolen.
  void idctcol(short *blk);
  //! Stolen.
  void idct(short *blk);
};



#endif   // __MXCPC_FRAMEWISEMXPEGSCANDECODERSOFTWAREONLY_H__
